/* soundmosaic, Copyright (c) 2001-2007 Steven Hazel <sah@thalassocracy.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 */
#include "graphics.h"
#include "config.h"

#if defined(HAVE_LIBX11) && \
    defined(HAVE_X11_XLIB_H) && \
    defined(HAVE_X11_XOS_H) && \
    defined (HAVE_X11_XUTIL_H)

/* marginal hack to draw the matches on an X window */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#define WIDTH 250
#define HEIGHT_DIVISOR 500

Display *dpy;
int screen;
Window root;
Window win;
GC gc;

void graphics_init()
{
  unsigned long black, white;
  char *display_name = getenv("DISPLAY");

  dpy = XOpenDisplay(display_name);
  if (!dpy) {
    fprintf(stderr, "unable to open display \"%s\"\n",
            XDisplayName(display_name));
    return;
  }

  screen = DefaultScreen(dpy);
  black = BlackPixel(dpy, screen);
  white = WhitePixel(dpy, screen);

  win = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 0, 0,
                            (WIDTH * 2) + 5, UINT16_MAX / HEIGHT_DIVISOR,
                            5, black, white);

  XSetStandardProperties(dpy, win, "soundmosaic", "soundmosaic",
                         None, NULL, 0, NULL);

  XSelectInput(dpy, win, ExposureMask);

  gc = XCreateGC(dpy, win, 0, 0);

  XSetBackground(dpy, gc, white);
  XSetForeground(dpy, gc, black);

  XClearWindow(dpy, win);
  XMapRaised(dpy, win);
  XSync(dpy, False);
}


void graphics_close()
{
  if (!dpy) {
    return;
  }
  XFreeGC(dpy, gc);
  XDestroyWindow(dpy, win);
  XCloseDisplay(dpy);
}


void graphics_draw_match(int16_t *target_tile,
                         int16_t *source_tile,
                         int32_t tile_size)
{
  int i;
  float step = (float)tile_size / (float)WIDTH;

  if (!dpy) {
    return;
  }

  if (step < 1) {
    step = 1;
  }

  XClearWindow(dpy, win);

  /* draw the first waveform */
  for (i = 0; i < tile_size; i = (float)i + step) {
    assert(i/step < WIDTH);
    XDrawLine(dpy, win, gc,
              i/step,
              (INT16_MAX/HEIGHT_DIVISOR),
              i/step,
              (INT16_MAX/HEIGHT_DIVISOR)-(target_tile[i]/HEIGHT_DIVISOR));
  }

  /* draw a vertical divider */
  for (i = 0; i < 5; i++) {
    XDrawLine(dpy, win, gc,
              (tile_size/step) + i,
              0,
              (tile_size/step) + i,
              (UINT16_MAX/HEIGHT_DIVISOR));
  }

  /* draw the second waveform */
  for (i = 0; i < tile_size; i = (float)i + step) {
    XDrawLine(dpy, win, gc,
              (i/step) + (tile_size/step) + 5,
              (INT16_MAX/HEIGHT_DIVISOR),
              (i/step) + (tile_size/step) + 5,
              (INT16_MAX/HEIGHT_DIVISOR) - (source_tile[i]/HEIGHT_DIVISOR));
  }

  XSync(dpy, False);
}

#else

void graphics_init()
{
}


void graphics_close()
{
}


void graphics_draw_match(int16_t *target_tile,
                         int16_t *source_tile,
                         int32_t tile_size)
{
}

#endif
